<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;  
use Illuminate\Support\Facades\Hash; 
use Illuminate\Support\Facades\Auth;
use App\Models\Client;
use App\Models\Company;
use App\Models\Card;

use Illuminate\Http\Response;

class ClientController extends Controller
{
     public function index(Request $request)
    {

         // Start the query builder for clients
        $clientsQuery = Client::query();

        // Initialize variables for the filters
        $reference = $request->input('reference') ?? null;
        $forename = $request->input('forename')?? null;
        $surname = $request->input('surname')?? null;
        $phone = $request->input('phone')?? null;
        $postcode = $request->input('postcode')?? null;
        $cardnumber = $request->input('cardnumber')?? null;

        // Apply filters if they are provided
        if ($reference) {
            $clientsQuery->where('reference', 'like', '%' . $reference . '%');
        }

        if ($forename) {
            $clientsQuery->where('forename', 'like', '%' . $forename . '%');
        }

        if ($surname) {
            $clientsQuery->where('surname', 'like', '%' . $surname . '%');
        }

        if ($phone) {
            $clientsQuery->where('phone', 'like', '%' . $phone . '%');
        }

        if ($postcode) {
            $clientsQuery->where('postcode', 'like', '%' . $postcode . '%');
        }

        if ($cardnumber) { 
            $clientsQuery->whereHas('card', function ($query) use ($cardnumber) {
                $query->where('number', 'like', '%' . $cardnumber . '%');
            });
        }


        if($request->export){


            $fileName = 'client_report.csv';  
            $client_datas = $clientsQuery->get();
     

            $headers = array(
                "Content-type"        => "text/csv",
                "Content-Disposition" => "attachment; filename=$fileName",
                "Pragma"              => "no-cache",
                "Cache-Control"       => "must-revalidate, post-check=0, pre-check=0",
                "Expires"             => "0"
            );

            $columns = array(  'Company Name','Reference','Salutation', 'First Name','Last Name','Email','Phone','Birthday','Address','City','Country','Postcode','Payment Method','Password','Remark');

            $callback = function() use($client_datas, $columns) {
                $file = fopen('php://output', 'w');
                fputcsv($file, $columns);
 
                foreach ($client_datas as $key => $client_data) {

                    $row['Company Name']  = companyDetail($client_data->company_id)->name ?? null ; 
                    $row['Reference']  = $client_data->reference ?? null ; 
                    $row['Salutation']  = Salutation($client_data->salutation) ?? null ; 
                    $row['First Name']  = $client_data->forename ?? null ; 
                    $row['Last Name']  = $client_data->surname ?? null ; 
                    $row['Email']  = $client_data->email ?? null ; 
                    $row['Phone']  = $client_data->phone ?? null ; 
                    $row['Birthday']  = $client_data->birth_date ?? null ; 
                    $row['Address']  = $client_data->address ?? null ; 
                    $row['City']  = $client_data->city ?? null ; 
                    $row['Country']  = $client_data->country ?? null ; 
                    $row['Postcode']  = $client_data->postcode ?? null ; 
                    $row['Payment Method']  = paymentMethod($client_data->payment_method) ?? null ;
                    $row['Password']  = $client_data->password ?? null ; 
                    $row['Remark']  = $client_data->remark ?? null ; 
                     
 
                    fputcsv($file, array($row['Company Name'],$row['Reference'],$row['Salutation'],$row['First Name'],$row['Last Name'],$row['Email'],$row['Phone'],$row['Birthday'],$row['Address'],$row['City'],$row['Country'],$row['Postcode'],$row['Payment Method'],$row['Password'],$row['Remark']));
                } 
                fclose($file);
            };

            return response()->stream($callback, 200, $headers);

        }

        // Paginate the results
        $clients = $clientsQuery->orderBy('id', 'desc')->paginate(10);
 

        // dd($clients->card->first()->holder_name);
        return view('admin.client.allclients',compact("clients","reference","forename","surname","phone","postcode","cardnumber")); 
    }


    public function index2()
    {

        $clients = Client::query();
        $reference = null;
        $forename = null;
        $surname = null;
        $phone = null;
        $postcode = null;
        $cardnumber = null;


         if ($request->reference) {
            $clients->where('reference', $request->reference);
            $reference = $request->reference;
        }

        if ($request->forename) {
            $clients->where('forename', $request->forename);
            $forename = $request->forename;
        }


        if ($request->phone) {
            $clients->where('phone', $request->phone);
            $phone = $request->phone;
        }


        if ($request->postcode) {
            $clients->where('postcode', $request->postcode);
            $postcode = $request->postcode;
        }


        if ($request->cardnumber) {
            $clients->where('cardnumber', $request->cardnumber);
            $cardnumber = $request->cardnumber;
        }

 
        $clients->orderBy('id', 'desc')->paginate(10);


        // $clients = Client::paginate(10); 

        // dd($clients->card->first()->holder_name);
        return view('admin.client.allclients',compact("clients")); 
    }
 
    public function create()
    {
        $companies = Company::all(); 
        return view('admin.client.new',compact("companies"));    
    }
 
    public function store(Request $request)
    {
 

         $message = [
            'company_id.required' => 'The company_id field is required.', 
            'forename.required' => 'The forename field is required.', 
            'surname.required' => 'The surname field is required.', 
            'email.required' => 'The email field is required.', 
            'phone.required' => 'The phone field is required.', 
            'birthday.required' => 'The birthday field is required.', 
            'country.required' => 'The country field is required.', 
            'city.required' => 'The city field is required.', 
            'town.required' => 'The town field is required.', 
            'postcode.required' => 'The postcode field is required.',  
            'address.required' => 'The address field is required.', 
            'payment_method.required' => 'The Payment Method field is required.',  
        ];

        $validatedData = $request->validate([ 
           'company_id' => 'required',
           'email' => 'required|email|max:255',
           'phone' => 'required|numeric|digits_between:1,15',
           'forename' => 'required|string',
           'surname' => 'required|string',
           'birthday' => 'required',
           'country' => 'required|string',
           'city' => 'required|string',
           'town' => 'required|string',
           'postcode' => 'required|string', 
           'address' => 'required|string',
           'payment_method' => 'required|numeric', 
        ], $message);



        // dd($request->all());

        $Client = new Client();

        $Client->company_id = $request->company_id;
        $Client->reference = $request->reference;
        $Client->salutation = $request->salutation;
        $Client->forename = $request->forename;
        $Client->surname = $request->surname;
        $Client->email = $request->email;
        $Client->birth_date = $request->birthday;
        $Client->phone = $request->phone;
        $Client->landline = $request->landline;
        $Client->country = $request->country;
        $Client->city = $request->city;
        $Client->town = $request->town;
        $Client->postcode = $request->postcode;
        $Client->address = $request->address; 
        $Client->payment_method = $request->payment_method; 
        $Client->password = $request->password;  
        $Client->save(); 

        return redirect()->route('admin.clients');
    }


    public function edit($id)
    {
        // dd('test');
        $client = Client::find($id);
        $companies = Company::all(); 
        return view('admin.client.edit',compact('client','companies'));
    }

 
    public function view($id)
    { 
        $client = Client::find($id); 
        $cards = Card::where('client_id',$id)->get(); 
         return view('admin.client.view',compact('client','cards'));
    }
 
    public function update(Request $request, $id)
    {
        $message = [
            'company_id.required' => 'The company_id field is required.', 
            'forename.required' => 'The forename field is required.', 
            'surname.required' => 'The surname field is required.', 
            'email.required' => 'The email field is required.', 
            'phone.required' => 'The phone field is required.', 
            'birthday.required' => 'The birthday field is required.', 
            'country.required' => 'The country field is required.', 
            'city.required' => 'The city field is required.', 
            'town.required' => 'The town field is required.', 
            'postcode.required' => 'The postcode field is required.',  
            'address.required' => 'The address field is required.', 
            'payment_method.required' => 'The Payment Method field is required.',
        ];

        $validatedData = $request->validate([ 
           'company_id' => 'required',
           'email' => 'required|email|max:255',
           'phone' => 'required|numeric|digits_between:1,15',
           'forename' => 'required|string',
           'surname' => 'required|string',
           'birthday' => 'required',
           'country' => 'required|string',
           'city' => 'required|string',
           'town' => 'required|string',
           'postcode' => 'required|string', 
           'address' => 'required|string',
           'payment_method' => 'required|numeric',
        ], $message);



        // dd($request->all());

        $Client = Client::find($id);

        $Client->company_id = $request->company_id;
        $Client->reference = $request->reference;
        $Client->salutation = $request->salutation;
        $Client->forename = $request->forename;
        $Client->surname = $request->surname;
        $Client->email = $request->email;
        $Client->birth_date = $request->birthday;
        $Client->phone = $request->phone;
        $Client->landline = $request->landline;
        $Client->country = $request->country;
        $Client->city = $request->city;
        $Client->town = $request->town;
        $Client->postcode = $request->postcode;
        $Client->address = $request->address; 
        $Client->payment_method = $request->payment_method; 
        $Client->password = $request->password;   
        $Client->update(); 

        return redirect()->route('admin.clients');
    }

     public function remark_update(Request $request, $id)
    {

        // dd($request->all());

          $message = [
            'remark.required' => 'The remark field is required.',  
        ];

        $validatedData = $request->validate([
           'remark' => 'required', 
        ], $message);

        
         
             $Client = Client::find($id);  
             $Client->remark = $request->remark; 
             $Client->save();

        return redirect()->back()->with('success', 'Card deleted successfully.'); 
    }
 
    public function destroy($id)
    { 
        $Client = Client::find($id);
        $Client->delete();
        return redirect()->route('admin.clients');

    }

   
}
