<?php

namespace App\Http\Controllers\admin;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;  
use Illuminate\Support\Facades\Hash; 
use Illuminate\Support\Facades\Auth;

use PhpOffice\PhpSpreadsheet\Spreadsheet;
use PhpOffice\PhpSpreadsheet\Writer\Xlsx;
use PhpOffice\PhpSpreadsheet\Style\Fill;
use PhpOffice\PhpSpreadsheet\Style\Font;
use PhpOffice\PhpSpreadsheet\Style\Border;
use PhpOffice\PhpSpreadsheet\Style\Alignment;


use App\Models\Client;
use App\Models\Company;
use App\Models\Card;
use App\Models\Policy;
use App\Models\Appliance;



class PolicyController extends Controller
{
    public function index(Request $request)
    {
  
        $policyQuery = Policy::query();



            // Pre Renewal Report
            if ($request->export == 1) {
                $fileName = 'Pre_renewal_report.xlsx';
                $pre_renewal_data = $policyQuery->get();

            // Create a new Spreadsheet object
                $spreadsheet = new Spreadsheet();
                $sheet = $spreadsheet->getActiveSheet();

            // Define column headers
                $columns = [
                    'Policy Number', 'Start date', 'Expiry Date', 'Policy Duration', 'Policy Type',
                    'Transaction Type', 'Policy holder Title', 'First Name', 'Last Name',
                    'Address Line 1', 'City/Town', 'Post Code', 'Make of Appliance',
                    'Type of Appliance', 'Gross Premium incl. IPT', 'Gross Premium EXCL. IPT',
                    'Tax/Levy %', 'IPT Amount', 'Commission Amount'
                ];

            // Add column headers with styling
                $headerStyle = [
                    'font' => [
                        'bold' => true,
                        'size' => 12,
                    'color' => ['rgb' => 'FFFFFF'], // White text color
                ],
                'fill' => [
                    'fillType' => Fill::FILL_SOLID,
                    'color' => ['rgb' => '1f4e78'] // Black background color
                ],
                'alignment' => [
                    'horizontal' => Alignment::HORIZONTAL_CENTER,
                    'vertical' => Alignment::VERTICAL_CENTER,
                     'wrapText' => true,
                ],
                'borders' => [
                    'allBorders' => [
                        'borderStyle' => Border::BORDER_THIN,
                        'color' => ['rgb' => '000000']
                    ]
                ]
            ];

            // Set column headers
            $sheet->fromArray($columns, NULL, 'A1');
            $sheet->getStyle('A1:S1')->applyFromArray($headerStyle);

            // Add data rows
            $rowIndex = 2; // Start from row 2 because row 1 has headers

            foreach ($pre_renewal_data as $pre_renewal) {
                $client = clientDetail($pre_renewal->client_id);
                $appliance = applianceDetail($pre_renewal->appliance_id);

                $row = [
                    $pre_renewal->plan_number ?? null,
                    $pre_renewal->start_date ?? null,
                    $pre_renewal->end_date ?? null,
                    Duration($pre_renewal->start_date, $pre_renewal->end_date) ?? null,
                    $pre_renewal->policy_type ?? null,
                    paymentMethod($client->payment_method) ?? null,
                    $client->title ?? null,
                    $client->forename ?? null,
                    $client->surname ?? null,
                    $client->address ?? null,
                    $client->city ?? null,
                    $client->postcode ?? null,
                    $appliance->manufacturer ?? null,
                    $appliance->type ?? null,
                    $pre_renewal->gross_premium_ipt ?? null,
                    $pre_renewal->gross_premium ?? null,
                    $pre_renewal->tax ?? null,
                    $pre_renewal->ipt_amount ?? null,
                    $pre_renewal->commission_amount ?? null,
                ];

                $sheet->fromArray($row, NULL, 'A' . $rowIndex++);
                }

                // Create Xlsx writer and save the file
                $writer = new Xlsx($spreadsheet);

                // Create a custom temp directory if needed
                $customTempDir = storage_path('app/temp');
                if (!file_exists($customTempDir)) {
                    mkdir($customTempDir, 0777, true);
                }

                // Generate a unique temporary file name
                $temp_file = tempnam($customTempDir, 'phpxltmp');

                // Error handling for file creation
                if ($temp_file === false) {
                    return response()->json(['error' => 'Failed to create temporary file.'], 500);
                }

                // Save the file
                try {
                    $writer->save($temp_file);
                } catch (\Exception $e) {
                    return response()->json(['error' => 'Failed to save the Excel file: ' . $e->getMessage()], 500);
                }

                // Return the response as an Excel file
                return response()->download($temp_file, $fileName)->deleteFileAfterSend(true);

            } 
  
            
            // Policy Doc
            if ($request->export == 2) {
                $fileName = 'policy_doc_report.xlsx';
                $policies_data = $policyQuery->get();

                // Create a new Spreadsheet object
                $spreadsheet = new Spreadsheet();
                $sheet = $spreadsheet->getActiveSheet();

                // Define column headers
                $columns = [
                    'Company Name','Salutation', 'Customer First Name','Customer Last Name','Billing Address','Billing City','Billing Country','Billing Postcode','Plan Number','Appliance Type','Manufacturer','Serial Number','Model','Appliance class','Renewal date ','Payment frequency','Recurring Payment Amount','Payment Method','Password'
                ];

                // Add column headers with styling
                $headerStyle = [
                    'font' => [
                        'bold' => true,
                        'size' => 12,
                        'color' => ['rgb' => '000000'], // White text color
                    ],
                    
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                        'vertical' => Alignment::VERTICAL_CENTER,
                    ], 
                ];

                // Set column headers
                $sheet->fromArray($columns, NULL, 'A1');
                $sheet->getStyle('A1:S1')->applyFromArray($headerStyle);

                // Add data rows
                $rowIndex = 2; // Start from row 2 because row 1 has headers

                foreach ($policies_data as $policy_data) {
 

                    $row = [ 

                      companyDetail(clientDetail($policy_data->client_id)->company_id)->name ?? null , 
                      Salutation(clientDetail($policy_data->client_id)->salutation) ?? null , 
                      clientDetail($policy_data->client_id)->forename ?? null , 
                      clientDetail($policy_data->client_id)->surname ?? null , 
                      clientDetail($policy_data->client_id)->address ?? null , 
                      clientDetail($policy_data->client_id)->city ?? null , 
                      clientDetail($policy_data->client_id)->country ?? null , 
                      clientDetail($policy_data->client_id)->postcode ?? null , 
                      $policy_data->plan_number ?? null , 
                      applianceDetail($policy_data->appliance_id)->type ?? null , 
                      applianceDetail($policy_data->appliance_id)->manufacturer ?? null , 
                      applianceDetail($policy_data->appliance_id)->serial_no ?? null , 
                      applianceDetail($policy_data->appliance_id)->model ?? null , 
                      applianceDetail($policy_data->appliance_id)->class ?? null , 
                      $policy_data->end_date ?? null , 
                      paymentFrequency($policy_data->payment_frequency) ?? null , 
                      $policy_data->recurring_payment_amount ?? null ,
                      paymentMethod(clientDetail($policy_data->client_id)->payment_method) ?? null ,
                      clientDetail($policy_data->client_id)->password ?? null , 


                    ];

                    $sheet->fromArray($row, NULL, 'A' . $rowIndex++);
                }

                // Create Xlsx writer and save the file
                $writer = new Xlsx($spreadsheet);

                // Create a custom temp directory if needed
                $customTempDir = storage_path('app/temp');
                if (!file_exists($customTempDir)) {
                    mkdir($customTempDir, 0777, true);
                }

                // Generate a unique temporary file name
                $temp_file = tempnam($customTempDir, 'phpxltmp');

                // Error handling for file creation
                if ($temp_file === false) {
                    return response()->json(['error' => 'Failed to create temporary file.'], 500);
                }

                // Save the file
                try {
                    $writer->save($temp_file);
                } catch (\Exception $e) {
                    return response()->json(['error' => 'Failed to save the Excel file: ' . $e->getMessage()], 500);
                }

                // Return the response as an Excel file
                return response()->download($temp_file, $fileName)->deleteFileAfterSend(true);

            } 



            // declarations
            if ($request->export == 3) {

                $fileName = 'Declaration.xlsx';
                $pre_renewal_data = $policyQuery->get();

                // Create a new Spreadsheet object
                $spreadsheet = new Spreadsheet();
                $sheet = $spreadsheet->getActiveSheet();

                // Define column headers for the detailed report
                $columns = [
                    'Company Name', 'Policy Number', 'Inception Date','Expiry Date','UW Year', 'Policy Duration', 'Payment Terms','Policy Type','Policy Holder Name', 'Policy Holder Surname', 'Policy Holder Address - First Line', 'Policy Holder Address - Town','Policy Holder Address - Postcode', 'Gross Premium Receivable incl. IPT', 'Gross Premium excl. IPT', 'Tax/Levy %','IPT Amount £', 'Commission %', 'Commission Amount £','Margin Payable', 'Claims Fund', 'Total Remitted','Bastion Net Premium'
                ];

                // Define summary headers
                $summaryHeaders = [
                    'Facility', 'Coverholder', 'Month', 'Currency','Total Payable'
                ];
 
 
                
                // Calculate summary data dynamically
                $totalPolicies = $pre_renewal_data->count();
                $totalGrossPremiumIpt = $pre_renewal_data->sum('gross_premium_ipt');
                $totalGrossPremiumExclIpt = $pre_renewal_data->sum('gross_premium');
                $averageTaxLevy = $pre_renewal_data->avg('tax');

                // Prepare summary data for insertion
                $summaryData = [
                    $totalPolicies,
                    $totalGrossPremiumIpt,
                    $totalGrossPremiumExclIpt,
                    $averageTaxLevy,
                    $averageTaxLevy
                ];



                // Add summary headers with styling
                $summaryHeaderStyle = [
                    'font' => [
                        'bold' => true,
                        'size' => 11,
                        'color' => ['rgb' => '000000'],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'color' => ['rgb' => '99cd00']
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                        'vertical' => Alignment::VERTICAL_CENTER,
                        'wrapText' => true, 
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000']
                        ]
                    ]
                ];



                // Add detailed column headers with styling
                $headerStyle = [
                    'font' => [
                        'bold' => true,
                        'size' => 11,
                        'color' => ['rgb' => 'FFFFFF'],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'color' => ['rgb' => '000000']
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                        'vertical' => Alignment::VERTICAL_CENTER,
                        'wrapText' => true,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000']
                        ]
                    ]
                ];

                  $headerStyle2 = [
                    'font' => [
                        'bold' => true,
                        'size' => 11,
                        'color' => ['rgb' => '000000'],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'color' => ['rgb' => 'fafa00']
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                        'vertical' => Alignment::VERTICAL_CENTER,
                        'wrapText' => true,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000']
                        ]
                    ]
                ];

                // Set summary headers
                $sheet->fromArray($summaryHeaders, NULL, 'A1');
                $sheet->getStyle('A1:E1')->applyFromArray($summaryHeaderStyle);


                // Set summary data
                $sheet->fromArray($summaryData, NULL, 'A2');
                $sheet->getStyle('A2:E2')->applyFromArray($summaryHeaderStyle);

                // Leave some rows between summary and detailed headers
                $currentRow = 6; // Row index for the detailed headers

                // Set detailed column headers
                $sheet->fromArray($columns, NULL, 'A' . $currentRow);
                $sheet->getStyle('A' . $currentRow . ':H' . $currentRow)->applyFromArray($headerStyle);
                $sheet->getStyle('H' . $currentRow . ':W' . $currentRow)->applyFromArray($headerStyle2);

  

                // Add data rows
                $rowIndex = $currentRow + 1; // Start from row below the headers

                foreach ($pre_renewal_data as $pre_renewal) {
                    $client = clientDetail($pre_renewal->client_id);
                    $appliance = applianceDetail($pre_renewal->appliance_id);  

                    $row = [
                        companyDetail($client->company_id)->name ?? null , 
                        $pre_renewal->plan_number ?? null,
                        $pre_renewal->start_date ?? null,
                        $pre_renewal->end_date ?? null,
                        Year($pre_renewal->start_date) ?? null, 
                        Duration($pre_renewal->start_date, $pre_renewal->end_date) ?? null,
                        $pre_renewal->payment_term ?? null,
                        $pre_renewal->policy_type ?? null, 
                        $client->forename ?? null,
                        $client->surname ?? null,
                        $client->address ?? null,
                        $client->city ?? null,
                        $client->postcode ?? null,
                        $pre_renewal->gross_premium_ipt ?? null,
                        $pre_renewal->gross_premium ?? null,
                        $pre_renewal->tax ?? null,
                        $pre_renewal->ipt_amount ?? null,
                        $pre_renewal->commission_amount ?? null,
                        $pre_renewal->commission_amount ?? null,
                        $pre_renewal->margin_payable ?? null,
                        $pre_renewal->claims_fund ?? null,
                        $pre_renewal->total_remitted ?? null, 
                        $pre_renewal->bastion_net_premium ?? null, 


                    ];

                    $sheet->fromArray($row, NULL, 'A' . $rowIndex++);
                }

                // Create Xlsx writer and save the file
                $writer = new Xlsx($spreadsheet);

                // Create a custom temp directory if needed
                $customTempDir = storage_path('app/temp');
                if (!file_exists($customTempDir)) {
                    mkdir($customTempDir, 0777, true);
                }

                // Generate a unique temporary file name
                $temp_file = tempnam($customTempDir, 'phpxltmp');

                // Error handling for file creation
                if ($temp_file === false) {
                    return response()->json(['error' => 'Failed to create temporary file.'], 500);
                }

                // Save the file
                try {
                    $writer->save($temp_file);
                } catch (\Exception $e) {
                    return response()->json(['error' => 'Failed to save the Excel file: ' . $e->getMessage()], 500);
                }

                // Return the response as an Excel file
                return response()->download($temp_file, $fileName)->deleteFileAfterSend(true);
            }


            // Refunds
            if ($request->export == 4) {

                $fileName = 'Refund.xlsx';
                $pre_renewal_data = $policyQuery->get();

                // Create a new Spreadsheet object
                $spreadsheet = new Spreadsheet();
                $sheet = $spreadsheet->getActiveSheet();

                // Define column headers for the detailed report
                $columns = [
                    'Company Name', 'Policy Number', 'Inception Date','Expiry Date','Policy Duration','Appliance Type','Gross Premium Receivable incl. IPT', 'Gross Premium excl. IPT', 'Tax/Levy %','IPT Amount £', 'Commission %', 'Commission Amount £','Margin Payable', 'Claims Fund', 'Total Remitted','Bastion Net Premium','Refund Date','Amount Refunded'
                ];

                // Define summary headers
                $summaryHeaders = [
                    'Facility', 'Coverholder', 'Month', 'Currency','Total Payable'
                ];
 
 
                
                // Calculate summary data dynamically
                $totalPolicies = $pre_renewal_data->count();
                $totalGrossPremiumIpt = $pre_renewal_data->sum('gross_premium_ipt');
                $totalGrossPremiumExclIpt = $pre_renewal_data->sum('gross_premium');
                $averageTaxLevy = $pre_renewal_data->avg('tax');

                // Prepare summary data for insertion
                $summaryData = [
                    $totalPolicies,
                    $totalGrossPremiumIpt,
                    $totalGrossPremiumExclIpt,
                    $averageTaxLevy,
                    $averageTaxLevy
                ];



                // Add summary headers with styling
                $summaryHeaderStyle = [
                    'font' => [
                        'bold' => true,
                        'size' => 11,
                        'color' => ['rgb' => '000000'],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'color' => ['rgb' => '99cd00']
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                        'vertical' => Alignment::VERTICAL_CENTER,
                        'wrapText' => true, 
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000']
                        ]
                    ]
                ];



                // Add detailed column headers with styling
                $headerStyle = [
                    'font' => [
                        'bold' => true,
                        'size' => 11,
                        'color' => ['rgb' => 'FFFFFF'],
                    ],
                    'fill' => [
                        'fillType' => Fill::FILL_SOLID,
                        'color' => ['rgb' => '000000']
                    ],
                    'alignment' => [
                        'horizontal' => Alignment::HORIZONTAL_CENTER,
                        'vertical' => Alignment::VERTICAL_CENTER,
                        'wrapText' => true,
                    ],
                    'borders' => [
                        'allBorders' => [
                            'borderStyle' => Border::BORDER_THIN,
                            'color' => ['rgb' => '000000']
                        ]
                    ]
                ];

                   

                // Set summary headers
                $sheet->fromArray($summaryHeaders, NULL, 'A1');
                $sheet->getStyle('A1:E1')->applyFromArray($summaryHeaderStyle);


                // Set summary data
                $sheet->fromArray($summaryData, NULL, 'A2');
                $sheet->getStyle('A2:E2')->applyFromArray($summaryHeaderStyle);

                // Leave some rows between summary and detailed headers
                $currentRow = 6; // Row index for the detailed headers

                // Set detailed column headers
                $sheet->fromArray($columns, NULL, 'A' . $currentRow);
                $sheet->getStyle('A' . $currentRow . ':R' . $currentRow)->applyFromArray($headerStyle); 

  

                // Add data rows
                $rowIndex = $currentRow + 1; // Start from row below the headers

                foreach ($pre_renewal_data as $pre_renewal) {
                    $client = clientDetail($pre_renewal->client_id);
                    $appliance = applianceDetail($pre_renewal->appliance_id);  
 

                    $row = [
                        companyDetail($client->company_id)->name ?? null , 
                        $pre_renewal->plan_number ?? null,
                        $pre_renewal->start_date ?? null,
                        $pre_renewal->end_date ?? null, 
                        Duration($pre_renewal->start_date, $pre_renewal->end_date) ?? null, 
                        $appliance->type ?? null,
                        $pre_renewal->gross_premium_ipt ?? null,
                        $pre_renewal->gross_premium ?? null,
                        $pre_renewal->tax ?? null,
                        $pre_renewal->ipt_amount ?? null,
                         $pre_renewal->commission_amount ?? null,
                        $pre_renewal->commission_amount ?? null,
                        $pre_renewal->margin_payable ?? null,
                        $pre_renewal->claims_fund ?? null,
                        $pre_renewal->total_remitted ?? null, 
                        $pre_renewal->bastion_net_premium ?? null, 
                        $pre_renewal->commission_amount ?? null,
                        $pre_renewal->commission_amount ?? null, 

                    ];

                    $sheet->fromArray($row, NULL, 'A' . $rowIndex++);
                }

                // Create Xlsx writer and save the file
                $writer = new Xlsx($spreadsheet);

                // Create a custom temp directory if needed
                $customTempDir = storage_path('app/temp');
                if (!file_exists($customTempDir)) {
                    mkdir($customTempDir, 0777, true);
                }

                // Generate a unique temporary file name
                $temp_file = tempnam($customTempDir, 'phpxltmp');

                // Error handling for file creation
                if ($temp_file === false) {
                    return response()->json(['error' => 'Failed to create temporary file.'], 500);
                }

                // Save the file
                try {
                    $writer->save($temp_file);
                } catch (\Exception $e) {
                    return response()->json(['error' => 'Failed to save the Excel file: ' . $e->getMessage()], 500);
                }

                // Return the response as an Excel file
                return response()->download($temp_file, $fileName)->deleteFileAfterSend(true);
            }
 

        $policies = Policy::paginate(10);
        return view('admin.policy.allpolicy',compact("policies")); 
    }

 
 
    public function create()
    {
        $companies = Policy::all(); 
        $clients = Client::all(); 
        $appliances = Appliance::all(); 
        return view('admin.policy.new',compact("clients","appliances"));    
    }
 
    public function store(Request $request)
    {
        
        // dd($request->all());
        // dd("working");

          $message = [
            'client_id.required' => 'The Client id field is required.', 
            'appliance_id.required' => 'The Appliance id field is required.', 
            'policy_type.required' => 'The Policy Type field is required.', 
            // 'plan_number.required' => 'The Plan Number field is required.', 
            'start_date.required' => 'The Start Date field is required.', 
            'end_date.required' => 'The End Date field is required.', 
            'gross_premium_ipt.required' => 'The Gross Premium IPT field is required.', 
            'gross_premium.required' => 'The Gross Premium is required.', 
            'tax.required' => 'The Tax field is required.', 
            'ipt_amount.required' => 'The IPT Amount field is required.',  
            'commission_amount.required' => 'The Commission Amount field is required.', 
            'recurring_payment_amount.required' => 'The Recurring Payment Amount field is required.', 
            'payment_frequency.required' => 'The Payment Frequency field is required.', 
            'payment_term.required' => 'The Payment Terms field is required.', 
            
        ];

        $validatedData = $request->validate([ 
           'client_id' => 'required',
           'appliance_id' => 'required',
           'policy_type' => 'required|string|max:255',
           // 'plan_number' => 'required|string|max:255',
           'start_date' => 'required',
           'end_date' => 'required',
           'gross_premium_ipt' => 'required|numeric',
           'gross_premium' => 'required|numeric',
           'tax' => 'required|numeric',
           'ipt_amount' => 'required|numeric', 
           'commission_amount' => 'required|numeric',
           'recurring_payment_amount' => 'required|numeric',
           'payment_frequency' => 'required|numeric',
           'payment_term' => 'required|numeric',
           
        ], $message);



        // dd($request->all());
        // dd("working");

        $prefix = "DCP";
        $timestamp = date('ymdHis'); // Year (yy), Month (mm), Day (dd), Hour (HH), Minute (MM), Second (SS)
        $randomNumber = mt_rand(10000, 99999); // Generate a random number between 10000 and 99999

         $plan_number = $prefix . $timestamp . $randomNumber;

         // dd($plan_number);

        $Policy = new Policy();

        $Policy->client_id = $request->client_id;
        $Policy->appliance_id = $request->appliance_id;
        $Policy->policy_type = $request->policy_type;
        $Policy->plan_number = $plan_number;
        $Policy->start_date = $request->start_date;
        $Policy->end_date = $request->end_date;
        $Policy->gross_premium_ipt = $request->gross_premium_ipt;
        $Policy->gross_premium = $request->gross_premium;
        $Policy->tax = $request->tax;
        $Policy->ipt_amount = $request->ipt_amount;
        $Policy->commission_amount = $request->commission_amount;
        $Policy->recurring_payment_amount = $request->recurring_payment_amount;
        $Policy->payment_frequency = $request->payment_frequency; 
        $Policy->payment_term = $request->payment_term; 
        $Policy->margin_payable = $request->margin_payable; 
        $Policy->claims_fund = $request->claims_fund; 
        $Policy->total_remitted = $request->total_remitted; 
        $Policy->bastion_net_premium = $request->bastion_net_premium; 
        $Policy->save(); 

        return redirect()->route('admin.policies');
    }


    public function edit($id)
    {
        // dd('test');
        $policy = Policy::find($id);
        $clients = Client::all(); 
        $appliances = Appliance::all();  
        return view('admin.policy.edit',compact('policy','clients','appliances'));
    }

 
    public function view($id)
    { 
        $client = Policy::find($id); 
        $cards = Card::where('client_id',$id)->get(); 
         return view('admin.policy.view',compact('client','cards'));
    }
 
    public function update(Request $request, $id)
    {
        $message = [
            'client_id.required' => 'The Client id field is required.', 
            'appliance_id.required' => 'The Appliance id field is required.', 
            'policy_type.required' => 'The Policy Type field is required.', 
            // 'plan_number.required' => 'The Plan Number field is required.', 
            'start_date.required' => 'The Start Date field is required.', 
            'end_date.required' => 'The End Date field is required.', 
            'gross_premium_ipt.required' => 'The Gross Premium IPT field is required.', 
            'gross_premium.required' => 'The Gross Premium is required.', 
            'tax.required' => 'The Tax field is required.', 
            'ipt_amount.required' => 'The IPT Amount field is required.',  
            'commission_amount.required' => 'The Commission Amount field is required.', 
            'recurring_payment_amount.required' => 'The Recurring Payment Amount field is required.', 
            'payment_frequency.required' => 'The Payment Frequency field is required.',  
            'payment_term.required' => 'The Payment Terms field is required.',  
        ];

        $validatedData = $request->validate([ 
           'client_id' => 'required',
           'appliance_id' => 'required',
           'policy_type' => 'required|string|max:255',
           // 'plan_number' => 'required|string|max:255',
           'start_date' => 'required',
           'end_date' => 'required',
           'gross_premium_ipt' => 'required|numeric',
           'gross_premium' => 'required|numeric',
           'tax' => 'required|numeric',
           'ipt_amount' => 'required|numeric', 
           'commission_amount' => 'required|numeric',
           'recurring_payment_amount' => 'required|numeric',
           'payment_frequency' => 'required|numeric', 
           'payment_term' => 'required|numeric', 
        ], $message);



        // dd($request->all());

        $Policy = Policy::find($id);

        $Policy->client_id = $request->client_id;
        $Policy->appliance_id = $request->appliance_id;
        $Policy->policy_type = $request->policy_type;
        $Policy->plan_number = $Policy->plan_number;
        $Policy->start_date = $request->start_date;
        $Policy->end_date = $request->end_date;
        $Policy->gross_premium_ipt = $request->gross_premium_ipt;
        $Policy->gross_premium = $request->gross_premium;
        $Policy->tax = $request->tax;
        $Policy->ipt_amount = $request->ipt_amount;
        $Policy->commission_amount = $request->commission_amount;
        $Policy->recurring_payment_amount = $request->recurring_payment_amount;
        $Policy->payment_frequency = $request->payment_frequency; 
        $Policy->payment_term = $request->payment_term; 
        $Policy->payment_term = $request->payment_term; 
        $Policy->margin_payable = $request->margin_payable; 
        $Policy->claims_fund = $request->claims_fund; 
        $Policy->total_remitted = $request->total_remitted; 
        $Policy->bastion_net_premium = $request->bastion_net_premium; 
        $Policy->update(); 

        return redirect()->route('admin.policies');
    }

   
 
    public function destroy($id)
    { 
        $Policy = Policy::find($id);
        $Policy->delete();
        return redirect()->route('admin.policies');

    }
}
